#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import logging
import os
import sys
import time

from decentra_network.config import LOGS_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.lib.settings_system import the_settings


def get_logger(name):
    logger = logging.getLogger(name)
    level = logging.DEBUG if the_settings()["debug_mode"] else logging.INFO
    logger.setLevel(level)
    # create console handler and set level to debug
    ch = logging.StreamHandler()
    ch.setLevel(level)
    # create formatter
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    # add formatter to ch
    ch.setFormatter(formatter)
    # add ch to logger
    logger.addHandler(ch)
    # file
    main_folder = get_config()["main_folder"]
    fh = logging.FileHandler(os.path.join(main_folder, LOGS_PATH, f"{name}.log"))
    fh.setLevel(level)
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    return logger



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import copy
import json
import os
import time

from decentra_network.accounts.account import Account
from decentra_network.accounts.get_accounts import GetAccounts
from decentra_network.accounts.save_accounts import SaveAccounts
from decentra_network.blockchain.block.blocks_hash import SaveBlockshash
from decentra_network.config import TEMP_BLOCK_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.lib.log import get_logger
from decentra_network.node.unl import Unl
from decentra_network.transactions.transaction import Transaction

logger = get_logger("BLOCKCHAIN")


class Block:
    """
    Block class is most important class. It is responsible for
    resetting and saving blocks.

    You must give a creator of the block. This creator will
    own all the coins.
    """

    def __init__(
        self,
        creator,
        previous_hash="fb8b69c2276c8316c64a5d34b5f3063d1f8b8dc17cda7ee84fa1343978d464a9-f86b4d545fe18264dc489f5af6782b9f4986fe3a9bf03b3fec417df9e8fd97d4",
    ):
        self.coin_amount = 10000000
        self.first_time = True
        self.creator = creator
        self.genesis_time = int(time.time())
        self.start_time = int(time.time())
        self.block_time = 22

        self.newly = False

        self.previous_hash = previous_hash
        self.sequance_number = 0
        self.empty_block_number = 0

        self.validating_list = []
        self.transaction_fee = 0.02
        self.default_transaction_fee = 0.02
        self.default_optimum_transaction_number = 1
        self.default_increase_of_fee = 0.01
        self.transaction_delay_time = 60
        self.max_data_size = 1000000

        self.part_amount = 100000

        self.hash = None

        self.max_tx_number = 2
        self.minumum_transfer_amount = 1000

        self.round_1_time = 10
        self.round_1 = False

        self.round_2_starting_time = None
        self.round_2_time = 10
        self.round_2 = False

        self.consensus_timer = 0.50

        self.validated = False
        self.validated_time = None

        self.dowload_true_block = ""

    def reset_the_block(self, custom_nodes=None):
        """
        When the block is verified and if block have a transaction
        and if block have at least half of the max_tx_number transaction,it saves the block
        and makes the edits for the new block.
        """

        self.start_time = int(time.time())

        self.round_1 = False

        self.round_2_starting_time = None
        self.round_2 = False

        self.validated = False
        self.validated_time = None

        # Resetting the node candidate blocks.
        nodes = (Unl.get_as_node_type(Unl.get_unl_nodes())
                 if custom_nodes is None else custom_nodes)
        for node in nodes:
            node.candidate_block = None
            node.candidate_block_hash = None

        if len(self.validating_list) >= (self.max_tx_number / 2):
            block2 = copy.copy(self)
            # Resetting and setting the new elements.
            self.previous_hash = self.hash
            self.sequance_number = self.sequance_number + 1
            self.validating_list = []
            self.hash = None
            logger.info("New block created")
            logger.debug(self.__dict__)
            return [block2, self]
        else:
            logger.info(
                "New block not created because no transaction enought to create a new block"
            )
            self.empty_block_number += 1
            return False

    def dump_json(self):
        """
        Dumps the block as json.
        """
        temp_block = copy.copy(self)

        temp_validating_list = [
            transaction.dump_json()
            for transaction in temp_block.validating_list
        ]

        temp_block.validating_list = temp_validating_list
        return temp_block.__dict__

    @staticmethod
    def load_json(json_string):
        temp_validating_list = [
            Transaction.load_json(tx) for tx in json_string["validating_list"]
        ]

        the_block_json = json.loads(json.dumps(json_string))
        the_block_json["validating_list"] = temp_validating_list
        the_block = Block("Decentra-Network")
        the_block.__dict__ = the_block_json

        return the_block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2021 Decentra Network Developers
Copyright (c) 2021 Onur Atakan ULUSOY

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


def starting_text_centered() -> str:
    """
    Prints the STARTING message between space.
    """

    print(" ")
    print(" STARTING")
    print(" ")

    return "\nSTARTING\n"


def ended_text_centered() -> str:
    """
    Prints the ENDED message between space.
    """
    print(" ")
    print(" ENDED")
    print(" ")

    return "\nENDED\n"


def printcentertext(text) -> str:
    """
    Prints the text between spaces.

    Inputs:
      * text: A string.
    """

    print(" ")
    print(text)
    print(" ")

    return "\n" + text + "\n"


def banner_maker(sc_name, description, author):
    """
    Returns a string in the form of a banner.

    Inputs:
      * sc_name: Name of the script.
      * description: Description of the script.
      * author: Author of the script.
    """

    return (((((f"""Script Name    : {sc_name}""" + """\n""" +
                """Description    : """) + description) + """\n""") +
             """Author         : """) + author) + """\n"""


def question_maker(question_text=None, mode=None):
    """
    Returns a string in the form of a question.

    Inputs:
      * mode: A string ("main" or "sub")
    """

    if question_text is None:
        if mode == "main":
            question_text = "Please enter main option: "
        elif mode == "sub":
            question_text = "Please enter sub option: "
        elif mode == "anykeytocontinue":
            question_text = "Press any key to continue..."
        else:
            question_text = "Please enter main option: "

    return input(question_text)


def menu_maker(menu_number, menu_text) -> str:
    """
    Returns a string in the form of a menu.

    Inputs:
      * menu_number: A data for menu defination (ex. 1, "sc")
    """

    return f"{str(menu_number)}) {menu_text}" + "\n"


def quit_menu_maker(mode) -> str:
    """
    Returns a quit string by mode.

    Inputs:
      * mode: A string ("main" or "sub")
    """

    if mode == "main":
        quit_menu_maker_result = "\n0) Quit \n"
    elif mode == "sub":
        quit_menu_maker_result = "\n0) Quit sub menu \n"
    else:
        quit_menu_maker_result = "\n0) Quit \n"

    return quit_menu_maker_result


def menu_space() -> str:
    """
    Returns a string in the form of a space.
    """

    return "\n"


def menu_seperator() -> str:
    """
    Returns a string in the form of a seperator.
    """

    return "\n" + "*** \n" + "\n"


def menu_title(menu_title_text) -> str:
    """
    Returns a string in the form of a title.

    Inputs:
      * menu_title_text: A string.
    """

    return "\n" + "*** " + menu_title_text + " ***" + " \n" + "\n"



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from contextlib import suppress
from decentra_network.blockchain.block.block_main import Block
from decentra_network.blockchain.block.get_block import GetBlock
from decentra_network.lib.log import get_logger
from decentra_network.wallet.ellipticcurve.wallet_import import wallet_import

logger = get_logger("BLOCKCHAIN")


def CreateBlock(custom_TEMP_BLOCK_PATH=None):
    """
    If test mode is on, creates genesis block
    and send the connected nodes, if it is off,
    it calls get_block() function.
    """

    previous_hash = None

    with suppress(Exception):
        current_block = GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
        if current_block.hash is not None:
            previous_hash = current_block.hash
        else:
            previous_hash = current_block.previous_hash

    logger.info("Creating the genesis block and sending it to the connected nodes")

    if previous_hash is None:
        return Block(wallet_import(-1, 3))
    else:
        return Block(wallet_import(-1, 3), previous_hash=previous_hash)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from decentra_network.blockchain.block.block_main import Block
from decentra_network.lib.log import get_logger

logger = get_logger("CONSENSUS")


def true_time(block: Block) -> bool:
    the_time = (block.genesis_time + block.block_time + (
        (block.sequance_number + block.empty_block_number) * block.block_time))
    if int(time.time()) >= the_time:
        return True
    else:
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.blockchain.block.hash.accounts_hash import AccountsHash
from decentra_network.blockchain.block.hash.blocks_hash import BlocksHash
from decentra_network.blockchain.block.hash.tx_hash import TransactionsHash
from decentra_network.lib.mix.merkle_root import MerkleTree


def CalculateHash(block, part_of_blocks_hash, the_blocks_hash, the_accounts):
    """
    Calculates and returns the hash of the block.
    """

    # Transaction Hash
    tx_hash = TransactionsHash(block)

    # Blocks Hash
    blocks_hash = BlocksHash(block, part_of_blocks_hash, the_blocks_hash)

    # Account
    accounts_hash = AccountsHash(block, the_accounts)

    # Other Elements
    main_list = [
        block.previous_hash,
        str(block.sequance_number),
        str(block.empty_block_number),
        blocks_hash,
        accounts_hash,
        tx_hash,
        str(block.default_transaction_fee),
        str(block.default_increase_of_fee),
        str(block.default_optimum_transaction_number),
    ]

    the_hash = MerkleTree(main_list).getRootHash()

    return the_hash



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.accounts.get_accounts import GetAccounts
from decentra_network.wallet.ellipticcurve.wallet_import import Address


def GetSequanceNumber(user, account_list=None):
    user = Address(user)
    sequance_number = 0
    the_account_list = GetAccounts() if account_list is None else account_list
    the_account_list.execute(
        f"SELECT * FROM account_list WHERE address = '{user}'")
    for Accounts in the_account_list.fetchall():
        sequance_number = Accounts[1]
        return sequance_number
    return sequance_number



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os

from kivy.core.clipboard import Clipboard
from kivymd.toast import toast
from kivymd.uix.button import MDFlatButton
from kivymd.uix.gridlayout import MDGridLayout
from kivymd.uix.screen import MDScreen
from kivymd_extensions.sweetalert import SweetAlert

from decentra_network.blockchain.block.create_block import CreateBlock
from decentra_network.blockchain.block.save_block import SaveBlock
from decentra_network.consensus.consensus_main import consensus_trigger
from decentra_network.lib.perpetualtimer import perpetualTimer
from decentra_network.lib.settings_system import the_settings
from decentra_network.lib.status import Status
from decentra_network.node.server.server import server


class NodeScreen(MDScreen):
    pass


# Start Node Server


class start_node_server_Box(MDGridLayout):
    cols = 2


# End

# Add UNL Node


class add_unl_node_Box(MDGridLayout):
    cols = 2


# End


class NodeBox(MDGridLayout):
    cols = 2
    FONT_PATH = f"{os.environ['DECENTRA_ROOT']}/gui_lib/fonts/"

    # Start Node Server
    start_node_server_dialog = None

    def show_start_node_server_dialog(self):
        if not self.start_node_server_dialog:
            self.start_node_server_dialog = SweetAlert(
                title="Start Node Server",
                type="custom",
                auto_dismiss=False,
                content_cls=start_node_server_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_start_node_server_dialog,
                        font_size="18sp",
                        font_name=f"{self.FONT_PATH}RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        on_press=self.start_node_server_func,
                        font_size="18sp",
                        font_name=f"{self.FONT_PATH}RobotoCondensed-Bold",
                    ),
                ],
            )

        self.start_node_server_dialog.open()

    def get_start_node_server_dialog_text(self):
        text_list = []
        for obj in self.start_node_server_dialog.content_cls.children:
            for sub_obj in obj.children:
                text_list.append(sub_obj.text)
                sub_obj.text = ""

        return text_list

    def start_node_server_func(self, widget):
        text_list = self.get_start_node_server_dialog_text()
        ip = text_list[1]
        port = text_list[0]

        print(ip)
        print(port)

        server(ip, int(port))

        self.start_node_server_dialog.dismiss()

    def dismiss_start_node_server_dialog(self, widget):
        self.get_start_node_server_dialog_text()

        self.start_node_server_dialog.dismiss()

    def start_node_server(self):
        self.show_start_node_server_dialog()

    # End

    def stop_node_server(self):
        server.Server.stop()

    def connect_to_know_node(self):
        server.connectionfrommixdb()

    # Connect a Node

    connect_a_node_dialog = None

    def show_connect_a_node_dialog(self):
        if not self.connect_a_node_dialog:
            self.connect_a_node_dialog = SweetAlert(
                title="Connect a Node",
                type="custom",
                auto_dismiss=False,
                content_cls=start_node_server_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_connect_a_node_dialog,
                        font_size="18sp",
                        font_name=f"{self.FONT_PATH}RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        on_press=self.connect_a_node_func,
                        font_size="18sp",
                        font_name=f"{self.FONT_PATH}RobotoCondensed-Bold",
                    ),
                ],
            )

        self.connect_a_node_dialog.open()

    def get_connect_a_node_dialog_text(self):
        text_list = []
        for obj in self.connect_a_node_dialog.content_cls.children:
            for sub_obj in obj.children:
                text_list.append(sub_obj.text)
                sub_obj.text = ""

        return text_list

    def connect_a_node_func(self, widget):
        text_list = self.get_connect_a_node_dialog_text()
        ip = text_list[1]
        port = text_list[0]

        print(ip)
        print(port)

        server.Server.connect(ip, int(port))

        self.connect_a_node_dialog.dismiss()

    def dismiss_connect_a_node_dialog(self, widget):
        self.get_connect_a_node_dialog_text()

        self.connect_a_node_dialog.dismiss()

    def connect_a_node(self):
        self.show_connect_a_node_dialog()

    # End

    # Add UNL Node

    add_unl_node_dialog = None

    def show_add_unl_node_dialog(self):
        if not self.add_unl_node_dialog:
            self.add_unl_node_dialog = SweetAlert(
                title="Add UNL Node",
                type="custom",
                auto_dismiss=False,
                content_cls=add_unl_node_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_add_unl_node_dialog,
                        font_size="18sp",
                        font_name=f"{self.FONT_PATH}RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        on_press=self.add_unl_node_func,
                        font_size="18sp",
                        font_name=f"{self.FONT_PATH}RobotoCondensed-Bold",
                    ),
                ],
            )

        self.add_unl_node_dialog.open()

    def get_add_unl_node_dialog_text(self):
        text_list = []
        for obj in self.add_unl_node_dialog.content_cls.children:
            for sub_obj in obj.children:
                text_list.append(sub_obj.text)
                sub_obj.text = ""

        return text_list

    def add_unl_node_func(self, widget):
        text_list = self.get_add_unl_node_dialog_text()
        pubkey = text_list[0]

        print(pubkey)

        from decentra_network.node.unl import Unl

        Unl.save_new_unl_node(pubkey)

        self.add_unl_node_dialog.dismiss()

    def dismiss_add_unl_node_dialog(self, widget):
        self.get_add_unl_node_dialog_text()

        self.add_unl_node_dialog.dismiss()

    def add_unl_node(self):
        self.show_add_unl_node_dialog()

    def get_block(self):
        if the_settings()["test_mode"]:
            the_block = CreateBlock()
            SaveBlock(the_block)
            server.Server.send_block_to_other_nodes()
            perpetualTimer(the_block.consensus_timer, consensus_trigger)
        else:
            server.Server.send_me_full_block()

    def nd_id(self):
        Clipboard.copy(server.id)
        SweetAlert().fire(
            "The ID has been copied to your clipboard.",
            type="success",
        )

    def status(self):
        toast("Calculating...")
        status = Status()
        if status == "Good":
            SweetAlert().fire(
                "Good",
                type="success",
            )
        elif status == "Not bad":
            SweetAlert().fire("Not bad", type="info")
        elif status == "Bad":
            SweetAlert().fire("Not bad", type="question")
        elif status == "Very bad":
            SweetAlert().fire("Very bad", type="warning")
        elif status == "Not work":
            SweetAlert().fire("Not work", type="failure")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.blockchain.block.block_main import Block
from decentra_network.lib.log import get_logger
from decentra_network.transactions.transaction import Transaction

logger = get_logger("TRANSACTIONS")


def Check_Len(block: Block, transaction: Transaction):
    """
    Check if the transaction lenght is valid
    """

    if (block.max_data_size / block.max_tx_number) >= len(transaction.data):
        pass
    else:
        logger.debug("Transaction data len is not correct")
        return False

    if len(transaction.fromUser) == 120:
        pass
    else:
        logger.debug("The from user is not correct")
        return False

    if len(transaction.toUser) <= 40:
        pass
    else:
        logger.debug("The to user is not correct")
        return False

    decimal_amount = len(str(block.transaction_fee).split(".")[1])

    if type(transaction.amount) == float:
        if len(str(transaction.amount).split(".")[1]) <= decimal_amount:
            pass
        else:
            logger.debug(
                "The decimal amount of transaction.amount is not true.")
            return False

    if type(transaction.transaction_fee) == float:
        if len(str(
                transaction.transaction_fee).split(".")[1]) <= decimal_amount:
            pass
        else:
            logger.debug(
                "The decimal amount of transaction.transaction_fee is not true."
            )
            return False

    if transaction.amount <= block.coin_amount:
        pass
    else:
        logger.debug("The amount is not true")
        return False

    if transaction.transaction_fee <= block.coin_amount:
        pass
    else:
        logger.debug("The transaction fee is not true")
        return False

    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.blockchain.block.block_main import Block
from decentra_network.lib.log import get_logger
from decentra_network.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction
from decentra_network.transactions.my_transactions.save_to_my_transaction import \
    SavetoMyTransaction
from decentra_network.transactions.my_transactions.sended_transaction import \
    SendedTransaction
from decentra_network.transactions.my_transactions.validate_transaction import \
    ValidateTransaction
from decentra_network.wallet.ellipticcurve.wallet_import import \
    wallet_import_all

logger = get_logger("CONSENSUS")


def transactions_main(block: Block) -> list:
    """
    This function is responsible for the transactions of the block.
    Parameters:
        block: The block that is going to be validated.
    Returns:
        list: The list of the transactions that are going to be validated.
    """
    new_my_transactions_list = None
    my_address = wallet_import_all(3)
    my_public_key = wallet_import_all(0)
    custom_currently_list = GetMyTransaction()
    for tx in block.validating_list:
        if tx.toUser in my_address:
            new_my_transactions_list = SavetoMyTransaction(
                tx,
                validated=True,
                custom_currently_list=custom_currently_list)
        elif tx.fromUser in my_public_key:
            new_my_transactions_list = ValidateTransaction(
                tx, custom_currently_list=custom_currently_list)
            new_my_transactions_list = SendedTransaction(
                tx, custom_currently_list=custom_currently_list)
    return new_my_transactions_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.config import *


def save_config(config):
    """
    Saves the settings.
    """

    temp_folder = os.path.dirname(os.path.realpath(__file__))
    os.chdir(temp_folder)
    os.chdir("..")
    with open(CONFIG_PATH, "w") as config_file:
        json.dump(config, config_file, indent=4)


def create_and_save_the_configs():
    """
    Creates and saves configs.
    """

    temp_json = {"main_folder": os.path.join(os.path.dirname(__file__), "..")}
    save_config(temp_json)
    return temp_json


def get_config():
    """
    Returns the configs. If it doesn't exist, it creates,
    saves and returns.
    """

    if not os.path.exists(CONFIG_PATH):
        return create_and_save_the_configs()
    with open(CONFIG_PATH, "r") as config_file:
        return json.load(config_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.blockchain.block.block_main import Block
from decentra_network.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from decentra_network.consensus.rounds.round_1.round_1_main import \
    consensus_round_1
from decentra_network.consensus.rounds.round_2.round_2_main import \
    consensus_round_2
from decentra_network.lib.log import get_logger
from decentra_network.node.client.client import client
from decentra_network.node.server.server import server

logger = get_logger("CONSENSUS")


def ongoing_main(
    block: Block,
    custom_candidate_class: candidate_block = None,
    custom_unl_nodes: dict = None,
    custom_UNL_NODES_PATH: str = None,
    custom_server: server = None,
    custom_unl: client = None,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,
) -> Block:

    if not block.round_1:
        logger.debug("First round is starting")
        consensus_round_1(
            block,
            custom_candidate_class=custom_candidate_class,
            custom_unl_nodes=custom_unl_nodes,
            custom_UNL_NODES_PATH=custom_UNL_NODES_PATH,
            custom_server=custom_server,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        logger.debug("First round is done")
    elif not block.round_2:
        logger.debug("Second round is starting")
        consensus_round_2(
            block,
            candidate_class=custom_candidate_class,
            unl_nodes=custom_unl_nodes,
            custom_server=custom_server,
            custom_unl=custom_unl,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        logger.debug("Second round is done")
    return block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.accounts.account import Account
from decentra_network.accounts.save_accounts import SaveAccounts
from decentra_network.blockchain.block.blocks_hash import SaveBlockshash
from decentra_network.blockchain.block.blocks_hash import SaveBlockshash_part
from decentra_network.config import TEMP_BLOCK_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.lib.log import get_logger

logger = get_logger("BLOCKCHAIN")


def SaveBlock(
    block,
    custom_TEMP_BLOCK_PATH=None,
    custom_TEMP_ACCOUNTS_PATH=None,
    custom_TEMP_BLOCKSHASH_PATH=None,
    custom_TEMP_BLOCKSHASH_PART_PATH=None,
):
    """
    Saves the current block to the TEMP_BLOCK_PATH.
    """
    logger.info("Saving block to disk")
    if block.first_time:
        SaveAccounts(
            Account(block.creator, block.coin_amount),
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
        )
        SaveBlockshash(
            block.previous_hash,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
        )
        SaveBlockshash_part(
            [block.previous_hash],
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        block.first_time = False
    the_TEMP_BLOCK_PATH = (TEMP_BLOCK_PATH if custom_TEMP_BLOCK_PATH is None
                           else custom_TEMP_BLOCK_PATH)
    os.chdir(get_config()["main_folder"])
    with open(the_TEMP_BLOCK_PATH, "w") as block_file:
        json.dump(block.dump_json(), block_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from distutils.log import info

from decentra_network.config import MY_TRANSACTION_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction
from decentra_network.transactions.my_transactions.save_my_transaction import \
    SaveMyTransaction
from decentra_network.transactions.transaction import Transaction


def SendedTransaction(tx: Transaction,
                      custom_currently_list: list = None) -> list:
    """
    Set sendedn the transaction.
    Parameters:
        tx: The transaction that is going to be validated.
    Returns:
        The list of the my transactions.
    """

    custom_currently_list = (GetMyTransaction()
                             if custom_currently_list is None else
                             custom_currently_list)
    for i in custom_currently_list:
        if i[0].signature == tx.signature:
            i[2] = True
    SaveMyTransaction(custom_currently_list)
    return custom_currently_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.blockchain.block.block_main import Block
from decentra_network.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from decentra_network.consensus.rounds.round_1.checks.candidate_blocks.candidate_blocks_main import \
    candidate_blocks_check
from decentra_network.consensus.rounds.round_1.checks.time.time_difference.time_difference_main import \
    time_difference_check
from decentra_network.lib.log import get_logger

logger = get_logger("CONSENSUS_FIRST_ROUND")


def round_check(block: Block, candidate_class: candidate_block,
                unl_nodes: dict) -> bool:
    if not candidate_blocks_check(candidate_class, unl_nodes):
        return False
    if not time_difference_check(block=block):
        return False

    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.blockchain.block.block_main import Block
from decentra_network.config import TEMP_BLOCK_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.lib.log import get_logger

logger = get_logger("BLOCKCHAIN")


def GetBlock(custom_TEMP_BLOCK_PATH=None):
    """
    Returns the block.
    """
    the_TEMP_BLOCK_PATH = (TEMP_BLOCK_PATH if custom_TEMP_BLOCK_PATH is None
                           else custom_TEMP_BLOCK_PATH)

    os.chdir(get_config()["main_folder"])
    with open(the_TEMP_BLOCK_PATH, "r") as block_file:
        the_block_json = json.load(block_file)
    result = Block.load_json(the_block_json)

    return result



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from decentra_network.lib.log import get_logger

logger = get_logger("CONSENSUS_SECOND_ROUND")


def candidate_blocks_hashes_check(candidate_class: candidate_block,
                                  unl_nodes: dict) -> bool:
    if len(candidate_class.candidate_block_hashes) >= (
        (len(unl_nodes) * 80) / 100):
        return True
    else:
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.blockchain.candidate_block.candidate_block_main import (
    candidate_block,
)
from decentra_network.node.unl import Unl


def GetCandidateBlocks(custom_nodes_list=None):
    """
    Collects candidate blocks and candidate block hashes
    from connected unl nodes and returns them in the
    candidate_block class
    """

    nodes = (
        Unl.get_as_node_type(Unl.get_unl_nodes())
        if custom_nodes_list is None
        else custom_nodes_list
    )

    the_candidate_blocks = []
    the_candidate_block_hashes = []

    for node in nodes:
        if node.candidate_block is not None:
            the_candidate_blocks.append(node.candidate_block)
        if node.candidate_block_hash is not None:
            the_candidate_block_hashes.append(node.candidate_block_hash)

    return candidate_block(the_candidate_blocks, the_candidate_block_hashes)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.config import PENDING_TRANSACTIONS_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.transactions.transaction import Transaction


def GetPending(custom_PENDING_TRANSACTIONS_PATH=None):
    the_PENDING_TRANSACTIONS_PATH = (PENDING_TRANSACTIONS_PATH if
                                     custom_PENDING_TRANSACTIONS_PATH is None
                                     else custom_PENDING_TRANSACTIONS_PATH)
    the_pending_list = []
    os.chdir(get_config()["main_folder"])
    for entry in os.scandir(the_PENDING_TRANSACTIONS_PATH):
        if entry.name != "README.md":
            with open(entry.path, "r") as my_transaction_file:
                the_pending_list.append(
                    Transaction.load_json(json.load(my_transaction_file)))
    return sorted(the_pending_list, key=lambda x: x.signature)


def GetPendingLen(custom_PENDING_TRANSACTIONS_PATH=None):
    the_PENDING_TRANSACTIONS_PATH = (PENDING_TRANSACTIONS_PATH if
                                     custom_PENDING_TRANSACTIONS_PATH is None
                                     else custom_PENDING_TRANSACTIONS_PATH)
    the_pending_list = []
    os.chdir(get_config()["main_folder"])
    for entry in os.scandir(the_PENDING_TRANSACTIONS_PATH):
        if entry.name != "README.md":
            the_pending_list.append(1)

    return len(the_pending_list)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.config import MY_TRANSACTION_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction


def SaveMyTransaction(transaction_list):
    """
    Saves the transaction_list to the transaction db.
    """

    if type(transaction_list) is list:
        new_dict = {
            tx[0].signature: {
                "tx": tx[0].dump_json(),
                "validated": tx[1],
                "sended": tx[2],
            }
            for tx in transaction_list
        }

        transaction_list = new_dict

    os.chdir(get_config()["main_folder"])
    with open(MY_TRANSACTION_PATH, "w") as my_transaction_file:
        json.dump(transaction_list, my_transaction_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import argparse
import os
import sys

from flask import Flask
from flask import jsonify
from flask import request
from waitress import serve
from waitress.server import create_server

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
from decentra_network.accounts.get_balance import GetBalance
from decentra_network.blockchain.block.create_block import CreateBlock
from decentra_network.blockchain.block.get_block import GetBlock
from decentra_network.blockchain.block.save_block import SaveBlock
from decentra_network.consensus.consensus_main import consensus_trigger
from decentra_network.lib.export import export_the_transactions
from decentra_network.lib.log import get_logger
from decentra_network.lib.perpetualtimer import perpetualTimer
from decentra_network.lib.safety import safety_check
from decentra_network.lib.settings_system import (d_mode_settings,
                                                  t_mode_settings,
                                                  the_settings)
from decentra_network.lib.status import Status
from decentra_network.node.server.server import server
from decentra_network.node.unl import Unl
from decentra_network.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction
from decentra_network.transactions.my_transactions.save_to_my_transaction import \
    SavetoMyTransaction
from decentra_network.transactions.send import send
from decentra_network.wallet.delete_current_wallet import delete_current_wallet
from decentra_network.wallet.ellipticcurve.wallet_create import wallet_create
from decentra_network.wallet.ellipticcurve.wallet_import import wallet_import
from decentra_network.wallet.print_wallets import print_wallets
from decentra_network.wallet.wallet_selector import wallet_selector

logger = get_logger("API")

app = Flask(__name__)

custom_block = None
custom_current_time = None
custom_sequence_number = None
custom_balance = None
custom_server = None

custom_TEMP_BLOCK_PATH = None
custom_TEMP_ACCOUNTS_PATH = None
custom_TEMP_BLOCKSHASH_PATH = None
custom_TEMP_BLOCKSHASH_PART_PATH = None

account_list = None

custom_wallet = None

custom_CONNECTED_NODES_PATH = None

custom_consensus_trigger = None
custom_consensus_trigger_result = None

custom_transactions = None
custom_MY_TRANSACTION_EXPORT_PATH = None

custom_UNL_NODES_PATH = None
custom_first_block = None
custom_new_block = None
custom_connections = None


@app.route("/wallet/print", methods=["GET"])
def print_wallets_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(print_wallets())


@app.route("/wallet/change/<number>", methods=["GET"])
def wallet_change_page(number):
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    wallet_selector(number)
    return jsonify(print_wallets())


@app.route("/wallet/create/<password>", methods=["GET"])
def create_wallet_page(password):
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    wallet_create(password)
    return jsonify(print_wallets())


@app.route("/wallet/delete", methods=["GET"])
def delete_wallets_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    delete_current_wallet()
    return jsonify(print_wallets())


@app.route("/send/", methods=["POST"])
def send_coin_data_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.form}")
    address = str(
        request.form["to_user"]) if "to_user" in request.form else None
    amount = float(
        request.form["amount"]) if "amount" in request.form else None
    data = str(request.form["data"]) if "data" in request.form else ""
    password = str(
        request.form["password"]) if "password" in request.form else None
    block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
             if custom_block is None else custom_block)
    send_tx = send(
        password,
        address,
        amount=amount,
        data=data,
        block=block,
        custom_current_time=custom_current_time,
        custom_sequence_number=custom_sequence_number,
        custom_balance=custom_balance,
    )
    if send_tx != False:
        SavetoMyTransaction(send_tx, sended=True)
        server.send_transaction(
            send_tx,
            custom_current_time=custom_current_time,
            custom_sequence_number=custom_sequence_number,
            custom_balance=custom_balance,
            custom_server=custom_server,
        )
        SaveBlock(
            block,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
    result = send_tx.dump_json() if send_tx != False else False
    return jsonify(result)


@app.route("/wallet/balance", methods=["GET"])
def balance_wallets_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    the_wallet = wallet_import(-1,
                               0) if custom_wallet is None else custom_wallet
    the_block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
                 if custom_block is None else custom_block)
    return jsonify(GetBalance(the_block, the_wallet,
                              account_list=account_list))


@app.route("/node/start/<ip>/<port>", methods=["GET"])
def node_start_page(ip, port):
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    server(str(ip), int(port))
    return jsonify("OK")


@app.route("/node/stop", methods=["GET"])
def node_stop_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    server.Server.stop()
    return jsonify("OK")


@app.route("/node/connect/<ip>/<port>", methods=["GET"])
def node_connect_page(ip, port):
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    server.Server.connect(str(ip), int(port))
    return jsonify("OK")


@app.route("/node/connectmixdb", methods=["GET"])
def node_connectmixdb_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    server.connectionfrommixdb(
        custom_server=custom_server,
        custom_CONNECTED_NODES_PATH=custom_CONNECTED_NODES_PATH,
    )
    return jsonify("OK")


# /node/newunl/?MFYw......
@app.route("/node/newunl/", methods=["GET"])
def node_newunl_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    Unl.save_new_unl_node(request.query_string.decode("utf-8"))
    return jsonify("OK")


@app.route("/node/id", methods=["GET"])
def node_id_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(server.id)


@app.route("/settings/test/on", methods=["GET"])
def settings_test_on_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    t_mode_settings(True)
    return jsonify("OK")


@app.route("/settings/test/off", methods=["GET"])
def settings_test_off_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    t_mode_settings(False)
    return jsonify("OK")


@app.route("/settings/debug/on", methods=["GET"])
def settings_debug_on_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    app.config["DEBUG"] = True
    d_mode_settings(True)
    return jsonify("OK")


@app.route("/settings/debug/off", methods=["GET"])
def settings_debug_off_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    app.config["DEBUG"] = False
    d_mode_settings(False)
    return jsonify("OK")


@app.route("/block/get", methods=["GET"])
def block_get_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    the_server = server.Server if custom_server is None else custom_server
    if the_settings()["test_mode"]:
        the_block = CreateBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
        SaveBlock(
            the_block,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        the_server.send_block_to_other_nodes()
        logger.info("Consensus timer is started")
        the_consensus_trigger = (consensus_trigger
                                 if custom_consensus_trigger is None else
                                 custom_consensus_trigger)
        trigger = perpetualTimer(the_block.consensus_timer,
                                 the_consensus_trigger)
        global custom_consensus_trigger_result
        custom_consensus_trigger_result = trigger
    else:
        the_server.send_me_full_block()
    return jsonify("OK")


@app.route("/export/transactions/csv", methods=["GET"])
def export_transaction_csv_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        export_the_transactions(
            custom_transactions=custom_transactions,
            custom_MY_TRANSACTION_EXPORT_PATH=custom_MY_TRANSACTION_EXPORT_PATH,
        ))


@app.route("/export/transactions/json", methods=["GET"])
def export_transaction_json_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        [f"{str(i[0].__dict__)} | {str(i[1])}" for i in GetMyTransaction()])


@app.route("/transactions/sended/validated", methods=["GET"])
def transaction_sended_validated_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        GetMyTransaction(sended=True, validated=True, turn_json=True))


@app.route("/transactions/sended/not_validated", methods=["GET"])
def transaction_sended_not_validated_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        GetMyTransaction(sended=True, validated=False, turn_json=True))





@app.route("/transactions/received", methods=["GET"])
def transaction_received_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        GetMyTransaction(sended=False, turn_json=True))

@app.route("/transactions/all", methods=["GET"])
def transaction_all_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        GetMyTransaction(turn_json=True))

@app.route("/status", methods=["GET"])
def status_page():
    logger.info(
        f"{request.remote_addr} {request.method} {request.url} {request.data}")
    return jsonify(
        Status(
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_UNL_NODES_PATH=custom_UNL_NODES_PATH,
            custom_first_block=custom_first_block,
            custom_new_block=custom_new_block,
            custom_connections=custom_connections,
            custom_transactions=custom_transactions,
        ))


@app.errorhandler(500)
def handle_exception(e):
    logger.exception(f"500: {e}")
    return jsonify("500"), 500


@app.errorhandler(404)
def page_not_found(e):
    logger.error(f"404: {e}")
    return jsonify("404"), 404


@app.errorhandler(405)
def method_not_allowed(e):
    logger.error(f"405: {e}")
    return jsonify("405"), 405


def start(port=None, test=False):
    """
    Start the API server.
    """

    parser = argparse.ArgumentParser(
        description=
        "This is an open source decentralized application network. In this network, you can develop and publish decentralized applications."
    )

    parser.add_argument("-p",
                        "--port",
                        default=8000,
                        type=int,
                        help="Add new UNL node")

    parser.add_argument(
        "-i",
        "--interface",
        type=str,
        help="Interface",
    )

    parser.add_argument(
        "-t",
        "--timeout",
        type=int,
        help="Timeout",
    )

    args = parser.parse_args()

    args.port = args.port if port is None else int(port)

    safety_check(args.interface, args.timeout)

    logger.info(f"Starting API on port {args.port}")
    result = (serve(app, host="0.0.0.0", port=args.port) if test is False else
              create_server(app, host="0.0.0.0", port=args.port))
    return result


if __name__ == "__main__":
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from kivy.clock import Clock
from kivy.uix.progressbar import ProgressBar
from kivymd.uix.screen import MDScreen


class DecentraWelcomeScreen(MDScreen):
    pb = ProgressBar()

    def __init__(self, *args, **kwargs):
        super(DecentraWelcomeScreen, self).__init__(*args, **kwargs)
        self.update_bar_trigger = Clock.schedule_interval(
            self.update_bar, 0.01)

    def update_bar(self, dt):
        if self.ids.pb.value < 100:
            self.ids.pb.value += 1
        else:
            self.update_bar_trigger.cancel()
            self.manager.current = "decentra root screen"



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from decentra_network.accounts.get_accounts import GetAccounts
from decentra_network.accounts.save_accounts import SaveAccounts
from decentra_network.blockchain.block.block_main import Block
from decentra_network.blockchain.block.blocks_hash import GetBlockshash
from decentra_network.blockchain.block.blocks_hash import GetBlockshash_part
from decentra_network.blockchain.block.blocks_hash import SaveBlockshash
from decentra_network.blockchain.block.blocks_hash import SaveBlockshash_part
from decentra_network.blockchain.block.hash.calculate_hash import CalculateHash
from decentra_network.blockchain.block.save_block import SaveBlock
from decentra_network.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from decentra_network.consensus.rounds.round_1.checks.checks_main import \
    round_check
from decentra_network.lib.log import get_logger
from decentra_network.node.get_candidate_blocks import GetCandidateBlocks
from decentra_network.node.server.server import server
from decentra_network.node.unl import Unl
from decentra_network.transactions.get_transaction import GetTransaction
from decentra_network.transactions.process_the_transaction import \
    ProccesstheTransaction

logger = get_logger("CONSENSUS_FIRST_ROUND")


def find_newly(block: Block, temp_validating_list: list) -> list:
    """
    Finds not validated new transaction in our block remove than and return as a new list
    """
    newly_added_list = []

    for my_validating_list in block.validating_list[:]:
        ok = any(
            (my_validating_list.signature == my_temp_validating_list.signature)
            for my_temp_validating_list in temp_validating_list[:])

        block.validating_list.remove(my_validating_list)
        if not ok:
            newly_added_list.append(my_validating_list)
    return newly_added_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import random

from decentra_network.blockchain.block.block_main import Block
from decentra_network.lib.log import get_logger
from decentra_network.node.client.client import client
from decentra_network.node.server.server import server
from decentra_network.node.unl import Unl

logger = get_logger("CONSENSUS_SECOND_ROUND")


def rescue_main(
    block: Block,
    candidate_block_hash: dict,
    custom_server: server = None,
    custom_unl: client = None,
) -> Block:
    sender = candidate_block_hash["sender"]
    logger.warning(
        f"Our block is not valid, the system will try to get true block from decentra_network.node {sender}"
    )
    block.dowload_true_block = sender
    unl_list = Unl.get_as_node_type([sender])
    the_server = server.Server if custom_server is None else custom_server
    the_unl_node = random.choice(
        unl_list) if custom_unl is None else custom_unl
    the_server.send_client(the_unl_node, {"action": "sendmefullblock"})
    return block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
from typing import List

from decentra_network.config import MY_TRANSACTION_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction
from decentra_network.transactions.my_transactions.save_my_transaction import \
    SaveMyTransaction
from decentra_network.transactions.transaction import Transaction


def SavetoMyTransaction(
    tx: Transaction,
    validated: bool = False,
    sended: bool = False,
    custom_currently_list: list = None,
) -> list:
    """
    Saves the transaction to the transaction db.
    Parameters:
        tx: The transaction that is going to be saved.
        validated: The boolean that sets if the transaction is validated or not.
        custom_currently_list: The list for custom situations.
    Returns:
        The list of the my transactions.
    """

    currently_list = (GetMyTransaction() if custom_currently_list is None else
                      custom_currently_list)
    tx_list = [tx, validated, sended]
    currently_list.append(tx_list)

    SaveMyTransaction(currently_list)

    return currently_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
import argparse

os.environ["KIVY_NO_ARGS"] = "1"
from kivy import Config
from kivy.lang import Builder
from kivymd.app import MDApp

from decentra_network.lib.config_system import get_config
from decentra_network.lib.log import get_logger
from decentra_network.lib.safety import safety_check

Config.set("graphics", "width", "700")
Config.set("graphics", "height", "450")
Config.set("graphics", "minimum_width", "700")
Config.set("graphics", "minimum_height", "450")
Config.set("input", "mouse", "mouse,disable_multitouch")

os.environ["DECENTRA_ROOT"] = get_config()["main_folder"]

KV_DIR = f"{os.environ['DECENTRA_ROOT']}/gui_lib/libs/kv/"

for kv_file in os.listdir(KV_DIR):
    with open(os.path.join(KV_DIR, kv_file), encoding="utf-8") as kv:
        Builder.load_string(kv.read())

KV = """
#:import FadeTransition kivy.uix.screenmanager.FadeTransition
#:import DecentraWelcomeScreen decentra_network.gui_lib.libs.baseclass.welcome_screen.DecentraWelcomeScreen
#:import DecentraRootScreen decentra_network.gui_lib.libs.baseclass.root_screen.DecentraRootScreen

ScreenManager:
    transition: FadeTransition()

    DecentraWelcomeScreen:
        name: "decentra register screen"

    DecentraRootScreen:
        name: "decentra root screen"

"""

logger = get_logger("GUI")


class GUI(MDApp):
    """
    An MDApp based  GUI class.
    GUI class consists of 2 elements:
      * title: Title of the app.
      * icon: icon of the app.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.title = "Decentra Network"
        self.icon = f"{os.environ['DECENTRA_ROOT']}/gui_lib/images/logo.ico"

    def build(self):
        """
        Some configurations.
        """

        self.theme_cls.primary_palette = "Green"
        FONT_PATH = f"{os.environ['DECENTRA_ROOT']}/gui_lib/fonts/"

        self.theme_cls.font_styles.update({
            "H1": [f"{FONT_PATH}RobotoCondensed-Light", 96, False, -1.5],
            "H2": [f"{FONT_PATH}RobotoCondensed-Light", 60, False, -0.5],
            "H3": [f"{FONT_PATH}Eczar-Regular", 48, False, 0],
            "H4": [f"{FONT_PATH}RobotoCondensed-Regular", 34, False, 0.25],
            "H5": [f"{FONT_PATH}RobotoCondensed-Regular", 24, False, 0],
            "H6": [f"{FONT_PATH}RobotoCondensed-Bold", 20, False, 0.15],
            "Subtitle1":
            [f"{FONT_PATH}RobotoCondensed-Regular", 16, False, 0.15],
            "Subtitle2":
            [f"{FONT_PATH}RobotoCondensed-Medium", 14, False, 0.1],
            "Body1": [f"{FONT_PATH}Eczar-Regular", 16, False, 0.5],
            "Body2": [f"{FONT_PATH}RobotoCondensed-Light", 14, False, 0.25],
            "Button": [f"{FONT_PATH}RobotoCondensed-Bold", 14, True, 1.25],
            "Caption": [f"{FONT_PATH}RobotoCondensed-Regular", 12, False, 0.4],
            "Overline": [f"{FONT_PATH}RobotoCondensed-Regular", 10, True, 1.5],
        })

        return Builder.load_string(KV)


def arguments():
    """
    This function parses the arguments and makes the directions.
    """

    parser = argparse.ArgumentParser(
        description=
        "This is an open source decentralized application network. In this network, you can develop and publish decentralized applications. Use the menu (-m) or GUI to gain full control and use the node, operation, etc."
    )

    parser.add_argument(
        "-i",
        "--interface",
        type=str,
        help="Interface",
    )

    parser.add_argument(
        "-t",
        "--timeout",
        type=int,
        help="Timeout",
    )

    args = parser.parse_args()

    safety_check(args.interface, args.timeout)

    GUI().run()


def start():
    """
    Start the GUI mode.
    """

    logger.info("Starting GUI mode")
    arguments()


if __name__ == "__main__":
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.lib.log import get_logger

logger = get_logger("TRANSACTIONS")


def Check_Type(transaction):
    """
    Check if the transaction type is valid
    """
    if isinstance(transaction.sequance_number, int):
        pass
    else:
        logger.debug("sequance_number is not int")
        return False

    if isinstance(transaction.signature, str):
        pass
    else:
        logger.debug("signature is not str")
        return False

    if isinstance(transaction.fromUser, str):
        pass
    else:
        logger.debug("fromUser is not str")
        return False

    if isinstance(transaction.toUser, str):
        pass
    else:
        logger.debug("toUser is not str")
        return False

    if isinstance(transaction.data, str):
        pass
    else:
        logger.debug("data is not str")
        return False

    if isinstance(transaction.amount, (int, float)):
        pass
    else:
        logger.debug("amount is not int&&float")
        return False

    if isinstance(transaction.transaction_fee, (int, float)):
        pass
    else:
        logger.debug("transaction_fee is not int&&float")
        return False

    if isinstance(transaction.transaction_time, (int)):
        pass
    else:
        logger.debug("transaction_time is not int")
        return False

    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from decentra_network.accounts.get_balance import GetBalance
from decentra_network.accounts.get_sequance_number import GetSequanceNumber
from decentra_network.lib.log import get_logger
from decentra_network.transactions.pending.get_pending import GetPending

logger = get_logger("TRANSACTIONS")


def Check_Datas(
    block,
    transaction,
    custom_current_time=None,
    custom_balance=None,
    custom_sequence_number=None,
    custom_PENDING_TRANSACTIONS_PATH=None,
):
    """
    Check if the transaction datas are valid
    """

    balance = (GetBalance(block, transaction.fromUser)
               if custom_balance is None else custom_balance)
    if balance >= (float(transaction.amount) +
                   float(transaction.transaction_fee)):
        pass
    else:
        logger.error("Balance is not valid")
        return False

    if transaction.amount >= block.minumum_transfer_amount:
        pass
    else:
        logger.error("Minimum transfer amount is not reached")
        return False

    if transaction.transaction_fee >= block.transaction_fee:
        pass
    else:
        logger.error("Transaction fee is not reached")
        return False

    pending_transactions = GetPending(
        custom_PENDING_TRANSACTIONS_PATH=custom_PENDING_TRANSACTIONS_PATH)
    for already_tx in pending_transactions + block.validating_list:
        if already_tx.signature == transaction.signature:
            logger.error("Transaction is already in the pending list")
            return False

    for tx in pending_transactions + block.validating_list:
        if (tx.fromUser == transaction.fromUser
                and tx.signature != transaction.signature):
            logger.info("Multiple transaction in one account")
            return False

    get_sequance_number = (GetSequanceNumber(transaction.fromUser)
                           if custom_sequence_number is None else
                           custom_sequence_number)
    if transaction.sequance_number == (get_sequance_number + 1):
        pass
    else:
        logger.error("Sequance number is not valid")
        return False

    current_time = (int(time.time())
                    if custom_current_time is None else custom_current_time)
    if (current_time -
            transaction.transaction_time) <= block.transaction_delay_time:
        pass
    else:
        logger.error("Transaction time is not valid")
        return False

    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import argparse
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
import time
from getpass import getpass

from decentra_network.accounts.get_balance import GetBalance
from decentra_network.blockchain.block.create_block import CreateBlock
from decentra_network.blockchain.block.get_block import GetBlock
from decentra_network.blockchain.block.save_block import SaveBlock
from decentra_network.config import MY_TRANSACTION_EXPORT_PATH
from decentra_network.consensus.consensus_main import consensus_trigger
from decentra_network.lib.export import export_the_transactions
from decentra_network.lib.log import get_logger
from decentra_network.lib.mix.mixlib import (banner_maker, menu_maker,
                                             menu_space, question_maker,
                                             quit_menu_maker)
from decentra_network.lib.perpetualtimer import perpetualTimer
from decentra_network.lib.safety import safety_check
from decentra_network.lib.settings_system import (d_mode_settings,
                                                  t_mode_settings,
                                                  the_settings)
from decentra_network.lib.status import Status
from decentra_network.node.server.server import server
from decentra_network.node.unl import Unl
from decentra_network.transactions.my_transactions.save_to_my_transaction import \
    SavetoMyTransaction
from decentra_network.transactions.print_transactions import PrintTransactions
from decentra_network.transactions.send import send
from decentra_network.wallet.delete_current_wallet import delete_current_wallet
from decentra_network.wallet.ellipticcurve.wallet_create import wallet_create
from decentra_network.wallet.ellipticcurve.wallet_import import wallet_import
from decentra_network.wallet.print_wallets import print_wallets
from decentra_network.wallet.wallet_selector import wallet_selector

logger = get_logger("CLI")


def show_menu():
    """
    Prints some information and the menu.
    """

    print(
        banner_maker(
            sc_name="Decentra Network",
            description=
            "This is an open source decentralized application network. In this network, you can develop and publish decentralized applications.",
            author="Decentra Network Developers",
        ))

    print(menu_space() +
          menu_maker(menu_number="pw", menu_text="Print Wallets") +
          menu_maker(menu_number="w", menu_text="Change Wallet") +
          menu_maker(menu_number="cw", menu_text="Create Wallet") +
          menu_maker(menu_number="dw", menu_text="Delete Wallet") +
          menu_space() + menu_maker(menu_number="sc", menu_text="Send Coin") +
          menu_maker(menu_number="scd", menu_text="Send Coin Data") +
          menu_space() +
          menu_maker(menu_number="gb", menu_text="Get Balance") +
          menu_space() +
          menu_maker(menu_number="ndstart", menu_text="Node Start") +
          menu_maker(menu_number="ndstop", menu_text="Node Stop") +
          menu_maker(menu_number="ndconnect", menu_text="Node Connect") +
          menu_maker(menu_number="ndconnectmixdb",
                     menu_text="Node Connect from mixdb") +
          menu_maker(menu_number="ndnewunl", menu_text="Add new UNL node") +
          menu_maker(menu_number="ndid", menu_text="Print my id") +
          menu_space() +
          menu_maker(menu_number="testmodeon", menu_text="Test mode ON") +
          menu_maker(menu_number="testmodeoff", menu_text="Test mode OF") +
          menu_maker(menu_number="debugmodeon", menu_text="Debug mode ON") +
          menu_maker(menu_number="debugmodeoff", menu_text="Debug mode OF") +
          menu_space() + menu_maker(menu_number="exptrcsv",
                                    menu_text="Export Transaction as CSV") +
          menu_maker(menu_number="returntrs",
                     menu_text="Export Transaction as CSV") + menu_space() +
          menu_maker(menu_number="status", menu_text="Prints the status") +
          menu_space() + menu_maker(menu_number="getblock",
                                    menu_text="Get block From Other Nodes") +
          menu_space())

    print(quit_menu_maker(mode="main"))


def menu():
    """
    The main structure of the cli mode, this function prints the menu,
    listens to the entries, makes the directions.
    """

    animation = [
        "[■□□□□□□□□□]",
        "[■■□□□□□□□□]",
        "[■■■□□□□□□□]",
        "[■■■■□□□□□□]",
        "[■■■■■□□□□□]",
        "[■■■■■■□□□□]",
        "[■■■■■■■□□□]",
        "[■■■■■■■■□□]",
        "[■■■■■■■■■□]",
        "[■■■■■■■■■■]",
    ]

    for item in animation:
        time.sleep(0.1)
        sys.stdout.write("\r" + item)
        sys.stdout.flush()

    while True:
        show_menu()
        choices_input = question_maker(mode="main")

        if choices_input == "pw":
            print_wallets()

        if choices_input == "w":
            wallet_selector(input("Please select wallet: "))

        if choices_input == "cw":
            wallet_create(getpass("Password: "))

        if choices_input == "dw" and input("Are you sure ? (y or n): ") == "y":
            delete_current_wallet()
        if choices_input == "sc":
            block = GetBlock()
            send_tx = send(
                getpass("Password: "),
                input("Please write receiver adress: "),
                amount=input("Coin Amount (ex. 1.0): "),
                block=block,
            )
            if send_tx != False:
                SavetoMyTransaction(send_tx, sended=True)
                server.send_transaction(send_tx)
                SaveBlock(block)
        if choices_input == "scd":
            block = GetBlock()
            send_tx = send(
                getpass("Password: "),
                input("Please write receiver adress: "),
                amount=input("Coin Amount (ex. 1.0): "),
                data=input("Data: "),
                block=block,
            )
            if send_tx != False:
                SavetoMyTransaction(send_tx, sended=True)
                server.send_transaction(send_tx)
                SaveBlock(block)
        if choices_input == "gb":
            GetBalance(GetBlock(), wallet_import(-1, 0))
        if choices_input == "help":
            show_menu()
        if choices_input == "ndstart":
            server(str(input("ip: ")), int(input("port: ")))
        if choices_input == "ndstop":
            server.Server.stop()
        if choices_input == "ndconnect":
            server.Server.connect(str(input("node ip: ")),
                                  int(input("node port: ")))

        if choices_input == "ndconnectmixdb":
            server.connectionfrommixdb()
        if choices_input == "ndnewunl":
            Unl.save_new_unl_node(input("Please write ID of the node: "))
        if choices_input == "ndid":
            print(server.id)
        if choices_input == "testmodeon":
            t_mode_settings(True)
        if choices_input == "testmodeoff":
            t_mode_settings(False)
        if choices_input == "debugmodeon":
            d_mode_settings(True)
        if choices_input == "debugmodeoff":
            d_mode_settings(False)

        if choices_input == "exptrcsv":
            if export_the_transactions():
                print(
                    f"CSV file created in {MY_TRANSACTION_EXPORT_PATH} directory"
                )
            else:
                print("You have not a transaction")

        if choices_input == "returntrs":
            PrintTransactions()

        if choices_input == "getblock":
            if the_settings()["test_mode"]:
                the_block = CreateBlock()
                SaveBlock(the_block)
                server.Server.send_block_to_other_nodes()
                logger.info("Consensus timer is started")
                perpetualTimer(the_block.consensus_timer, consensus_trigger)
            else:
                server.Server.send_me_full_block()

        if choices_input == "status":
            print(Status())

        if choices_input == "0":
            exit()


def arguments():
    """
    This function parses the arguments and makes the directions.
    """

    parser = argparse.ArgumentParser(
        description=
        "This is an open source decentralized application network. In this network, you can develop and publish decentralized applications. Use the menu (-m) or GUI to gain full control and use the node, operation, etc."
    )

    parser.add_argument("-pw",
                        "--printwallet",
                        action="store_true",
                        help="Print Wallets")

    parser.add_argument("-w", "--wallet", type=int, help="Change Wallet")

    parser.add_argument("-cw", "--createwallet", help="Create wallet")

    parser.add_argument("-dw",
                        "--deletewallet",
                        action="store_true",
                        help="Delete wallet")

    parser.add_argument("-gb",
                        "--getbalance",
                        action="store_true",
                        help="Get Balance")

    parser.add_argument("-ndnunl",
                        "--ndnewunl",
                        type=str,
                        help="Add new UNL node")

    parser.add_argument("-ndid",
                        "--ndid",
                        action="store_true",
                        help="Print my id")

    parser.add_argument("-tmon",
                        "--testmodeon",
                        action="store_true",
                        help="Test Mode On")
    parser.add_argument("-tmoff",
                        "--testmodeoff",
                        action="store_true",
                        help="Test Mode Off")

    parser.add_argument("-dmon",
                        "--debugmodeon",
                        action="store_true",
                        help="Debug Mode On")
    parser.add_argument("-dmoff",
                        "--debugmodeoff",
                        action="store_true",
                        help="Debug Mode Off")

    parser.add_argument(
        "-exptrcsv",
        "--exporttransactioncsv",
        action="store_true",
        help="Exports the transaction as csv",
    )

    parser.add_argument(
        "-returntrans",
        "--returntransactions",
        action="store_true",
        help="Exports the transaction as csv",
    )

    parser.add_argument("-st",
                        "--status",
                        action="store_true",
                        help="Exports the transaction as csv")

    parser.add_argument(
        "-m",
        "--menu",
        action="store_true",
        help="An optional boolean for open the menu.",
    )

    parser.add_argument(
        "-i",
        "--interface",
        type=str,
        help="Interface",
    )

    parser.add_argument(
        "-t",
        "--timeout",
        type=int,
        help="Timeout",
    )

    args = parser.parse_args()

    if len(sys.argv) < 2:
        parser.print_help()

    if args.printwallet:
        print_wallets()

    if args.getbalance:
        GetBalance(GetBlock(), wallet_import(-1, 0))

    if args.ndid:
        print(server.id)

    if args.testmodeon:
        t_mode_settings(True)
    if args.testmodeoff:
        t_mode_settings(False)
    if args.debugmodeon:
        d_mode_settings(True)
    if args.debugmodeoff:
        d_mode_settings(False)

    if args.exporttransactioncsv:
        if export_the_transactions():
            print(
                f"CSV file created in {MY_TRANSACTION_EXPORT_PATH} directory")
        else:
            print("You have not a transaction")

    if args.returntransactions:
        PrintTransactions()

    if args.status:
        print(Status())

    safety_check(args.interface, args.timeout)

    if args.ndnewunl is not None:
        Unl.save_new_unl_node(args.ndnewunl)

    if args.createwallet is not None:
        wallet_create(args.createwallet)

    if args.deletewallet:
        delete_current_wallet()

    if args.wallet is not None:
        wallet_selector(args.wallet)

    if args.menu:
        menu()


def start():
    """
    Start the CLI mode with arguments.
    """

    logger.info("Starting CLI mode")

    arguments()


if __name__ == "__main__":
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.config import MY_TRANSACTION_PATH
from decentra_network.lib.config_system import get_config
from decentra_network.transactions.transaction import Transaction


def GetMyTransaction(sended=None, validated=None, turn_json=False) -> list:
    """
    Returns the transaction db.
    """

    os.chdir(get_config()["main_folder"])

    if not os.path.exists(MY_TRANSACTION_PATH):
        return []

    the_transactions = []

    with open(MY_TRANSACTION_PATH, "r") as my_transaction_file:
        the_transactions_json = json.load(my_transaction_file)
        for transaction in list(the_transactions_json.values()):
            the_transactions.append([
                Transaction.load_json(transaction["tx"]),
                transaction["validated"],
                transaction["sended"],
            ])

    if sended is not None:
        the_transactions = [tx for tx in the_transactions if tx[2] == sended]

    if validated is not None:
        the_transactions = [
            tx for tx in the_transactions if tx[1] == validated
        ]

    if turn_json:
        the_transactions = {
            the_transactions.index(tx): {
                "transaction": tx[0].dump_json(),
                "validated": tx[1],
                "sended": tx[2],
            }
            for tx in the_transactions
        }

    return the_transactions



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from io import open

from setuptools import setup

setup(
    author="Decentra Network Developers",
    author_email="onur@decentranetwork.org",
    packages=["decentra_network"],
    name="decentra_network",
    version="0.30.3",
    url="https://github.com/Decentra-Network/Decentra-Network",
    description=
    "This is an open source decentralized application network. In this network, you can develop and publish decentralized applications.",
    keywords=[
        "python",
        "cryptography",
        "blockchain",
        "p2p",
        "python3",
        "cryptocurrency",
        "kivy",
        "coin",
        "copilot",
        "fba",
        "dapps",
        "p2p-network",
        "kivymd",
        "blokzinciri",
        "decentra-network",
        "githubcopilot",
        "blokzincir",
    ],
    long_description_content_type="text/markdown",
    long_description="".join(open("README.md", encoding="utf-8").readlines()),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "dngui = decentra_network.gui:start",
            "dncli = decentra_network.cli:start",
            "dnapi = decentra_network.api:start",
        ],
    },
    license="MPL-2.0",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Programming Language :: Python :: 3",
    ],
    python_requires=">=3.8",
)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.transactions.pending.get_pending import GetPendingLen


def ChangeTransactionFee(
    block,
    custom_pending_transaction_len=None,
    custom_PENDING_TRANSACTIONS_PATH=None,
):
    """
    Increase transaction fee by 0.01 DNC for each block.default_optimum_transaction_number argument
    """
    pending_transactions = (GetPendingLen(
        custom_PENDING_TRANSACTIONS_PATH=custom_PENDING_TRANSACTIONS_PATH)
                            if custom_pending_transaction_len is None else
                            custom_pending_transaction_len)
    total_len = len(block.validating_list) + pending_transactions
    if (total_len // block.default_optimum_transaction_number) != 0:
        increase = (total_len // block.default_optimum_transaction_number
                    ) * block.default_increase_of_fee
        block.transaction_fee += increase
    else:
        block.transaction_fee = block.default_transaction_fee



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time
from hashlib import sha256

from decentra_network.accounts.get_sequance_number import GetSequanceNumber
from decentra_network.lib.log import get_logger
from decentra_network.lib.settings_system import the_settings
from decentra_network.transactions.get_transaction import GetTransaction
from decentra_network.transactions.transaction import Transaction
from decentra_network.wallet.ellipticcurve.ecdsa import Ecdsa
from decentra_network.wallet.ellipticcurve.privateKey import PrivateKey
from decentra_network.wallet.ellipticcurve.wallet_import import wallet_import

from decentra_network.blockchain.block.get_block import GetBlock


logger = get_logger("TRANSACTIONS")


def send(
    password,
    to_user,
    amount=None,
    data="",
    block=None,
    custom_current_time=None,
    custom_sequence_number=None,
    custom_balance=None,
):
    """
    The main function for sending the transaction.

    Inputs:
        password: The password of the wallet.
        to_user: The address of the recipient.
        amount: The amount of the transaction.
        data: The data of the transaction.

    """
    block = block if block is not None else GetBlock()

    amount = amount if amount is not None else block.minumum_transfer_amount

    try:
        amount = float(amount)
    except ValueError:
        logger.exception("This is not float coin amount.")
        return False

    if amount < 0:
        logger.error("This is negative coin amount.")
        return False

    if (block.max_data_size / block.max_tx_number) < len(data):
        logger.error("The data is too long.")
        return False

    decimal_amount = len(str(block.transaction_fee).split(".")[1])
    if len(str(amount).split(".")[1]) > decimal_amount:
        logger.error(f"The amount of decimal places is more than {decimal_amount}.")
        return False

    if amount >= block.minumum_transfer_amount:
        if (
            wallet_import(int(the_settings()["wallet"]), 2)
            == sha256(password.encode("utf-8")).hexdigest()
        ):

            my_private_key = wallet_import(-1, 1, password)
            my_public_key = "".join(
                [
                    l.strip()
                    for l in wallet_import(-1, 0).splitlines()
                    if l and not l.startswith("-----")
                ]
            )

            sequance_number = GetSequanceNumber(my_public_key) + 1

            # Get the current fee
            transaction_fee = block.transaction_fee

            tx_time = int(time.time())
            the_transaction = Transaction(
                sequance_number,
                Ecdsa.sign(
                    (str(sequance_number) + my_public_key + str(to_user) + str(data))
                    + str(amount)
                    + str(transaction_fee)
                    + str(tx_time),
                    PrivateKey.fromPem(my_private_key),
                ).toBase64(),
                my_public_key,
                to_user,
                data,
                amount,
                transaction_fee,
                tx_time,
            )

            if GetTransaction(
                block,
                the_transaction,
                custom_current_time=custom_current_time,
                custom_sequence_number=custom_sequence_number,
                custom_balance=custom_balance,
            ):

                del my_private_key
                del password

                return the_transaction
            else:
                logger.error("The transaction is not valid.")
                return False

        else:
            logger.error("Password is not correct")
            return False
    else:
        logger.error(f"The amount is too low. minumum:{block.minumum_transfer_amount}")
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from kivy.properties import ColorProperty
from kivy.properties import StringProperty
from kivymd.theming import ThemableBehavior
from kivymd.uix.behaviors import RectangularRippleBehavior
from kivymd.uix.boxlayout import MDBoxLayout
from kivymd.uix.screen import MDScreen


class DecentraRootScreen(MDScreen):
    pass


class DecentraListItem(ThemableBehavior, RectangularRippleBehavior,
                       MDBoxLayout):
    text = StringProperty()
    secondary_text = StringProperty()
    tertiary_text = StringProperty()
    bar_color = ColorProperty((1, 0, 0, 1))


class DecentraSeeAllButton(RectangularRippleBehavior, MDBoxLayout):
    pass



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))

from decentra_network.lib.config_system import get_config


def CleanUp_tests():

    os.chdir(get_config()["main_folder"])
    for the_file in os.listdir("db/"):
        if the_file.startswith("test_"):
            if os.path.isfile(f"db/{the_file}"):
                with contextlib.suppress(PermissionError):
                    os.remove(f"db/{the_file}")

    for the_file in os.listdir(
            "db/test_SaveBlockstoBlockchainDB_GetBlockstoBlockchainDB/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(
                f"db/test_SaveBlockstoBlockchainDB_GetBlockstoBlockchainDB/{the_file}"
            )

    for the_file in os.listdir("db/test_finished_main/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(f"db/test_finished_main/{the_file}")

    for the_file in os.listdir("db/test_consensus_trigger_finished/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(f"db/test_consensus_trigger_finished/{the_file}")

    for the_file in os.listdir("db/connected_nodes_test_0/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(f"db/connected_nodes_test_0/{the_file}")
    for the_file in os.listdir("db/connected_nodes_test_1/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(f"db/connected_nodes_test_1/{the_file}")
    for the_file in os.listdir("db/connected_nodes_test_2/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(f"db/connected_nodes_test_2/{the_file}")

    for the_file in os.listdir("db/pending_transactions_test_1/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(f"db/pending_transactions_test_1/{the_file}")
    for the_file in os.listdir("db/pending_transactions_test_2/"):
        if the_file.endswith(".json") or the_file.endswith(".db"):

            os.remove(f"db/pending_transactions_test_2/{the_file}")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import json
import socket
import time
from threading import Thread

from decentra_network.lib.log import get_logger

logger = get_logger("NODE")


class client(Thread):

    def __init__(self, socket, address, node_id, server, test=False):
        Thread.__init__(self)
        self.server = server
        self.socket = socket
        self.host = address[0]
        self.port = address[1]
        self.id = node_id
        self.candidate_block = None
        self.candidate_block_hash = None

        self.running = True
        if not test:
            self.start()

    def run(self):
        self.socket.settimeout(10.0)
        while self.running:
            with contextlib.suppress(socket.timeout):
                data = self.socket.recv(6525)
                logger.debug(
                    f"NODE:{self.server.host}:{self.server.port} SOCK:{self.host}:{self.port} Received data"
                )
                data = data.decode("utf-8")
                with contextlib.suppress(json.decoder.JSONDecodeError):
                    data = json.loads(data)
                with contextlib.suppress(KeyError):
                    del data["buffer"]
                self.server.get_message(self, data)

            time.sleep(0.01)

    def stop(self):
        self.running = False
        self.socket.close()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import sys
from threading import Thread

import kivy
from kivy.app import App
from kivy.uix.label import Label

from decentra_network.api.main import start


class Decentra_Network_API(App):

    def build(self):
        the_api = Thread(target=start)
        the_api.start()
        return Label(text="Decentra-Network-API")


if __name__ == "__main__":
    Decentra_Network_API().run()
